// src/dsiUnit.js
import DSIParser from './dsiParser.js';

export class DSIUnit {
  constructor(dsiString) {
    const parser = DSIParser.getInstance();
    const result = parser.parse(dsiString);
    this.dsiString = result.original;
    this.tree = result.tree;
    this.warnings = result.warnings;
    this.nonDsiUnit = result.nonDsiUnit;
    // For non-DSI units (or empty string), treat them as "valid" for rendering purposes.
    this.valid = this.nonDsiUnit ? true : (this.warnings.length === 0);
    this.scaleFactor = 1.0;
  }

  toHTML(options = {}) {
    let htmlOutput = '';
    if (this.nonDsiUnit) {
      const content = `<span class="dsi-nonunit">${this.dsiString.startsWith('|') ? this.dsiString.slice(1) : this.dsiString}</span>`;
      htmlOutput = content;
    } else if (this.tree.length === 1) {
      htmlOutput = this.tree[0].map(node => node.toHTML(options)).join(options.oneLine ? " " : '<span class="dsi-mul">&nbsp;</span>');
    } else if (this.tree.length === 2) {
      if (options.oneLine) {
        const numerator = this.tree[0].map(node => node.toHTML(options)).join(" ");
        const denominator = this.tree[1].map(node => node.toHTML(options)).join(" ");
        htmlOutput = `${numerator} / ${denominator}`;
      } else {
        const numerator = this.tree[0].map(node => node.toHTML(options)).join('<span class="dsi-mul">&nbsp;</span>');
        const denominator = this.tree[1].map(node => node.toHTML(options)).join('<span class="dsi-mul">&nbsp;</span>');
        htmlOutput = `<span class="dsi-fraction"><span class="dsi-numerator">${numerator}</span><span class="dsi-denom">${denominator}</span></span>`;
      }
    } else {
      htmlOutput = this.tree.map(frac => frac.map(node => node.toHTML(options)).join(options.oneLine ? " " : '<span class="dsi-mul">&nbsp;</span>'))
          .join(options.oneLine ? " / " : '<span class="dsi-invalid">/</span>');
    }
    if (this.scaleFactor !== 1.0) {
      htmlOutput = `<span class="dsi-scale">${this.scaleFactor}&middot;</span>` + htmlOutput;
    }
    if (!this.nonDsiUnit && !this.valid) {
      htmlOutput = `<span class="dsi-error">${htmlOutput}</span>`;
    }
    let tooltip = "";
    if (!this.nonDsiUnit && this.warnings && this.warnings.length > 0) {
      const tooltipText = this.warnings.map(w => w.replace(/\\\\/g, '\\')).join("; ");
      tooltip = ` title="${tooltipText}"`;
    }
    return `<span class="dsi-unit-wrapper"${tooltip}>${htmlOutput}</span>`;
  }

  toString() {
    if (this.nonDsiUnit) return '|' + this.dsiString;
    return this.tree.map(frac => frac.map(node => node.toString()).join('')).join('\\per');
  }
}
